﻿/*
 * Copyright(c) 2023 John P. Doran
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish, 
 * distribute, sublicense, create a derivative work, and/or sell copies of the 
 * Software in any work that is designed, intended, or marketed for pedagogical or 
 * instructional purposes related to programming, coding, application development, 
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works, 
 * or sale is expressly withheld.
 *    
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
using UnityEngine;

public class PlayerController : MonoBehaviour
{
    // Variables
    private bool facingRight = true;
    [SerializeField] private float moveForce = 15f;
    [SerializeField] private float jumpForce = 250f;
    [SerializeField] private float maxSpeed = 10f;

    // Components
    private Animator anim;
    private Rigidbody2D rb2d;
    private SpriteRenderer sr;

    /// <summary>
    /// Initialization of needed components
    /// </summary>
    void Start()
    {
        anim = GetComponent<Animator>();
        rb2d = GetComponent<Rigidbody2D>();
        sr = GetComponent<SpriteRenderer>();
    }

    /// <summary>
    /// Physics update handles movement over time and if can jump when on ground
    /// </summary>
    void FixedUpdate()
    {
        // Find out if we are moving
        float horizMovement = Input.GetAxis("Horizontal");

        // Update the animator component
        anim.SetFloat("xSpeed", Mathf.Abs(horizMovement));

        // Limit the player's speed in the x axis
        if (Mathf.Abs(rb2d.velocity.x) < maxSpeed)
        {
            rb2d.AddForce(Vector2.right * horizMovement * moveForce * Time.deltaTime);
        }
    }

    /// <summary>
    /// Handles code that should instantly change, such as moving from left 
    /// to right or jumping.
    /// </summary>
    void Update()
    {
        float xMove = Input.GetAxis("Horizontal");

        // If we are going in a new direction, we need to flip our sprite
        if ((xMove > 0) && !facingRight)
        { 
            Flip(); 
        }
        else if ((xMove < 0) && facingRight)
        {
            Flip();
        }

        // Check if we should jump
        if (Input.GetButtonDown("Jump"))
        {
            anim.SetTrigger("Jump");
            rb2d.AddForce(new Vector2(0f, jumpForce));
        }
    }

    // Flip the sprite
    void Flip()
    {
        facingRight = !facingRight;
        sr.flipX = !facingRight;
    }
}